/* Copyright 2012, Google Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *    * Neither the name of Google Inc. nor the names of its
 *      contributors may be used to endorse or promote products derived
 *      from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * JEDEC Solid State Technology Association, formerly known as Joint Electron
 * Device Engineering Council (JEDEC) or Joint Electron Device Engineering
 * Councils, is the semiconductor engineering standardization body of the
 * Electronic Industries Alliance (EIA), a trade association that represents
 * all areas of the electronics industry in the United States.
 */

#include <stdlib.h>

#include "lib/string.h"

#include "jedec_id.h"

/*
 * JEDEC Standard Manufacturer's Identification Code
 * from JEP106U, October 2006
 * http://www.jedec.org/download/search/jep106U-01.pdf
 */
static const char *jedec_id1[] = {
	[1] = "AMD",
	[2] = "AMI",
	[3] = "Fairchild",
	[4] = "Fujitsu",
	[5] = "GTE",
	[6] = "Harris",
	[7] = "Hitachi",
	[8] = "Inmos",
	[9] = "Intel",
	[10] = "I.T.T.",
	[11] = "Intersil",
	[12] = "Monolithic Memories",
	[13] = "Mostek",
	[14] = "Freescale",
	[15] = "National",
	[16] = "NEC",
	[17] = "RCA",
	[18] = "Raytheon",
	[19] = "Conexant",
	[20] = "Seeq",
	[21] = "Philips Semi. (Signetics)",
	[22] = "Synertek",
	[23] = "Texas Instruments",
	[24] = "Toshiba",
	[25] = "Xicor",
	[26] = "Zilog",
	[27] = "Eurotechnique",
	[28] = "Mitsubishi",
	[29] = "Lucent (AT&T)",
	[30] = "Exel",
	[31] = "Atmel",
	[32] = "SGS/Thomson",
	[33] = "Lattice Semi.",
	[34] = "NCR",
	[35] = "Wafer Scale Integration",
	[36] = "IBM",
	[37] = "Tristar",
	[38] = "Visic",
	[39] = "Intl. CMOS Technology",
	[40] = "SSSI",
	[41] = "Microchip Technology",
	[42] = "Ricoh Ltd.",
	[43] = "VLSI",
	[44] = "Micron Technology",
	[45] = "Hyundai Electronics",
	[46] = "OKI Semiconductor",
	[47] = "ACTEL",
	[48] = "Sharp",
	[49] = "Catalyst",
	[50] = "Panasonic",
	[51] = "IDT",
	[52] = "Cypress",
	[53] = "DEC",
	[54] = "LSI Logic",
	[55] = "Zarlink",
	[56] = "UTMC",
	[57] = "Thinking Machine",
	[58] = "Thomson CSF",
	[59] = "Integrated CMOS(Vertex)",
	[60] = "Honeywell",
	[61] = "Tektronix",
	[62] = "Sun Microsystems",
	[63] = "SST",
	[64] = "ProMos/Mosel Vitelic (Mosel)",
	[65] = "Infineon",
	[66] = "Macronix",
	[67] = "Xerox",
	[68] = "Plus Logic",
	[69] = "SunDisk",
	[70] = "Elan Circuit Tech.",
	[71] = "European Silicon Str.",
	[72] = "Apple Computer",
	[73] = "Xilinx",
	[74] = "Compaq",
	[75] = "Protocol Engines",
	[76] = "SCI",
	[77] = "Seiko Instruments",
	[78] = "Samsung",
	[79] = "I3 Design System",
	[80] = "Klic",
	[81] = "Crosspoint Solutions",
	[82] = "Alliance Semiconductor",
	[83] = "Tandem",
	[84] = "Hewlett-Packard",
	[85] = "Intg. Silicon Solutions",
	[86] = "Brooktree",
	[87] = "New Media",
	[88] = "MHS Electronic",
	[89] = "Performance Semi.",
	[90] = "Winbond Electronic",
	[91] = "Kawasaki Steel",
	[92] = "Bright Micro",
	[93] = "TECMAR",
	[94] = "Exar",
	[95] = "PCMCIA",
	[96] = "LG Semiconductor",
	[97] = "Northern Telecom",
	[98] = "Sanyo",
	[99] = "Array Microsystems",
	[100] = "Crystal Semiconductor",
	[101] = "Analog Devices",
	[102] = "PMC-Sierra",
	[103] = "Asparix",
	[104] = "Convex Computer",
	[105] = "Quality Semiconductor",
	[106] = "Nimbus Technology",
	[107] = "Transwitch",
	[108] = "Micronas (ITT Intermetall)",
	[109] = "Cannon",
	[110] = "Altera",
	[111] = "NEXCOM",
	[112] = "QUALCOMM",
	[113] = "Sony",
	[114] = "Cray Research",
	[115] = "AMS (Austria Micro)",
	[116] = "Vitesse",
	[117] = "Aster Electronics",
	[118] = "Bay Networks (Synoptic)",
	[119] = "Zentrum",
	[120] = "TRW",
	[121] = "Thesys",
	[122] = "Solbourne Computer",
	[123] = "Allied-Signal",
	[124] = "Dialog",
	[125] = "Media Vision",
	[126] = "Level One Communication",
};

static const char *jedec_id2[] = {
	[1] = "Cirrus Logic",
	[2] = "National Instruments",
	[3] = "ILC Data Device",
	[4] = "Alcatel Mietec",
	[5] = "Micro Linear",
	[6] = "Univ. of NC",
	[7] = "JTAG Technologies",
	[8] = "BAE Systems",
	[9] = "Nchip",
	[10] = "Galileo Tech",
	[11] = "Bestlink Systems",
	[12] = "Graychip",
	[13] = "GENNUM",
	[14] = "VideoLogic",
	[15] = "Robert Bosch",
	[16] = "Chip Express",
	[17] = "DATARAM",
	[18] = "United Microelec Corp.",
	[19] = "TCSI",
	[20] = "Smart Modular",
	[21] = "Hughes Aircraft",
	[22] = "Lanstar Semiconductor",
	[23] = "Qlogic",
	[24] = "Kingston",
	[25] = "Music Semi",
	[26] = "Ericsson Components",
	[27] = "SpaSE",
	[28] = "Eon Silicon Devices",
	[29] = "Programmable Micro Corp",
	[30] = "DoD",
	[31] = "Integ. Memories Tech.",
	[32] = "Corollary Inc.",
	[33] = "Dallas Semiconductor",
	[34] = "Omnivision",
	[35] = "EIV(Switzerland)",
	[36] = "Novatel Wireless",
	[37] = "Zarlink (formerly Mitel)",
	[38] = "Clearpoint",
	[39] = "Cabletron",
	[40] = "Silicon Technology",
	[41] = "Vanguard",
	[42] = "Hagiwara Sys-Com",
	[43] = "Vantis",
	[44] = "Celestica",
	[45] = "Century",
	[46] = "Hal Computers",
	[47] = "Rohm Company Ltd.",
	[48] = "Juniper Networks",
	[49] = "Libit Signal Processing",
	[50] = "Mushkin Enhanced Memory (Ramtron)",
	[51] = "Tundra Semiconductor",
	[52] = "Adaptec Inc.",
	[53] = "LightSpeed Semi.",
	[54] = "ZSP Corp.",
	[55] = "AMIC Technology",
	[56] = "Adobe Systems",
	[57] = "Dynachip",
	[58] = "PNY Electronics",
	[59] = "Newport Digital",
	[60] = "MMC Networks",
	[61] = "T Square",
	[62] = "Seiko Epson",
	[63] = "Broadcom",
	[64] = "Viking Components",
	[65] = "V3 Semiconductor",
	[66] = "Flextronics",
	[67] = "Suwa Electronics",
	[68] = "Transmeta",
	[69] = "Micron CMS",
	[70] = "American Computer & Digital Components Inc",
	[71] = "Enhance 3000 Inc",
	[72] = "Tower Semiconductor",
	[73] = "CPU Design",
	[74] = "Price Point",
	[75] = "Maxim Integrated Product",
	[76] = "Tellabs",
	[77] = "Centaur Technology",
	[78] = "Unigen Corporation",
	[79] = "Transcend Information",
	[80] = "Memory Card Technology",
	[81] = "CKD Corporation Ltd.",
	[82] = "Capital Instruments, Inc.",
	[83] = "Aica Kogyo, Ltd.",
	[84] = "Linvex Technology",
	[85] = "MSC Vertriebs GmbH",
	[86] = "AKM Company, Ltd.",
	[87] = "Dynamem, Inc.",
	[88] = "NERA ASA",
	[89] = "GSI Technology",
	[90] = "Dane-Elec (C Memory)",
	[91] = "Acorn Computers",
	[92] = "Lara Technology",
	[93] = "Oak Technology, Inc.",
	[94] = "Itec Memory",
	[95] = "Tanisys Technology",
	[96] = "Truevision",
	[97] = "Wintec Industries",
	[98] = "Super PC Memory",
	[99] = "MGV Memory",
	[100] = "Galvantech",
	[101] = "Gadzoox Nteworks",
	[102] = "Multi Dimensional Cons.",
	[103] = "GateField",
	[104] = "Integrated Memory System",
	[105] = "Triscend",
	[106] = "XaQti",
	[107] = "Goldenram",
	[108] = "Clear Logic",
	[109] = "Cimaron Communications",
	[110] = "Nippon Steel Semi. Corp.",
	[111] = "Advantage Memory",
	[112] = "AMCC",
	[113] = "LeCroy",
	[114] = "Yamaha Corporation",
	[115] = "Digital Microwave",
	[116] = "NetLogic Microsystems",
	[117] = "MIMOS Semiconductor",
	[118] = "Advanced Fibre",
	[119] = "BF Goodrich Data.",
	[120] = "Epigram",
	[121] = "Acbel Polytech Inc.",
	[122] = "Apacer Technology",
	[123] = "Admor Memory",
	[124] = "FOXCONN",
	[125] = "Quadratics Superconductor",
	[126] = "3COM",
};

static const char *jedec_id3[] = {
	[1] = "Camintonn Corporation",
	[2] = "ISOA Incorporated",
	[3] = "Agate Semiconductor",
	[4] = "ADMtek Incorporated",
	[5] = "HYPERTEC",
	[6] = "Adhoc Technologies",
	[7] = "MOSAID Technologies",
	[8] = "Ardent Technologies",
	[9] = "Switchcore",
	[10] = "Cisco Systems, Inc.",
	[11] = "Allayer Technologies",
	[12] = "WorkX AG",
	[13] = "Oasis Semiconductor",
	[14] = "Novanet Semiconductor",
	[15] = "E-M Solutions",
	[16] = "Power General",
	[17] = "Advanced Hardware Arch.",
	[18] = "Inova Semiconductors GmbH",
	[19] = "Telocity",
	[20] = "Delkin Devices",
	[21] = "Symagery Microsystems",
	[22] = "C-Port Corporation",
	[23] = "SiberCore Technologies",
	[24] = "Southland Microsystems",
	[25] = "Malleable Technologies",
	[26] = "Kendin Communications",
	[27] = "Great Technology Microcomputer",
	[28] = "Sanmina Corporation",
	[29] = "HADCO Corporation",
	[30] = "Corsair",
	[31] = "Actrans System Inc.",
	[32] = "ALPHA Technologies",
	[33] = "Silicon Labratories, Inc. (Cygnal)",
	[34] = "Artesyn Technologies",
	[35] = "Align Manufacturing",
	[36] = "Peregrine Semiconductor",
	[37] = "Chameleon Systems",
	[38] = "Aplus Flash Technology",
	[39] = "MIPS Technologies",
	[40] = "Chrysalis ITS",
	[41] = "ADTEC Corporation",
	[42] = "Kentron Technologies",
	[43] = "Win Technologies",
	[44] = "Tachyon Semiconductor (ASIC Designs Inc)",
	[45] = "Extreme Packet Devices",
	[46] = "RF Micro Devices",
	[47] = "Siemens AG",
	[48] = "Sarnoff Corporation",
	[49] = "Itautec Philco SA",
	[50] = "Radiata Inc.",
	[51] = "Benchmark Elect. (AVEX)",
	[52] = "Legend",
	[53] = "SpecTek Incorporated",
	[54] = "Hi/fn",
	[55] = "Enikia Incorporated",
	[56] = "SwitchOn Networks",
	[57] = "AANetcom Incorporated",
	[58] = "Micro Memory Bank",
	[59] = "ESS Technology",
	[60] = "Virata Corporation",
	[61] = "Excess Bandwidth",
	[62] = "West Bay Semiconductor",
	[63] = "DSP Group",
	[64] = "Newport Communications",
	[65] = "Chip2Chip Incorporated",
	[66] = "Phobos Corporation",
	[67] = "Intellitech Corporation",
	[68] = "Nordic VLSI ASA",
	[69] = "Ishoni Networks",
	[70] = "Silicon Spice",
	[71] = "Alchemy Semiconductor",
	[72] = "Agilent Technologies",
	[73] = "Centillium Communications",
	[74] = "W.L. Gore",
	[75] = "HanBit Electronics",
	[76] = "GlobeSpan",
	[77] = "Element 14",
	[78] = "Pycon",
	[79] = "Saifun Semiconductors",
	[80] = "Sibyte, Incorporated",
	[81] = "MetaLink Technologies",
	[82] = "Feiya Technology",
	[83] = "I & C Technology",
	[84] = "Shikatronics",
	[85] = "Elektrobit",
	[86] = "Megic",
	[87] = "Com-Tier",
	[88] = "Malaysia Micro Solutions",
	[89] = "Hyperchip",
	[90] = "Gemstone Communications",
	[91] = "Anadigm (Anadyne)",
	[92] = "3ParData",
	[93] = "Mellanox Technologies",
	[94] = "Tenx Technologies",
	[95] = "Helix AG",
	[96] = "Domosys",
	[97] = "Skyup Technology",
	[98] = "HiNT Corporation",
	[99] = "Chiaro",
	[100] = "MCI Computer GMBH",
	[101] = "Exbit Technology A/S",
	[102] = "Integrated Technology Express",
	[103] = "AVED Memory",
	[104] = "Legerity",
	[105] = "Jasmine Networks",
	[106] = "Caspian Networks",
	[107] = "nCUBE",
	[108] = "Silicon Access Networks",
	[109] = "FDK Corporation",
	[110] = "High Bandwidth Access",
	[111] = "MultiLink Technology",
	[112] = "BRECIS",
	[113] = "World Wide Packets",
	[114] = "APW",
	[115] = "Chicory Systems",
	[116] = "Xstream Logic",
	[117] = "Fast-Chip",
	[118] = "Zucotto Wireless",
	[119] = "Realchip",
	[120] = "Galaxy Power",
	[121] = "eSilicon",
	[122] = "Morphics Technology",
	[123] = "Accelerant Networks",
	[124] = "Silicon Wave",
	[125] = "SandCraft",
	[126] = "Elpida",
};

static const char *jedec_id4[] = {
	[1] = "Solectron",
	[2] = "Optosys Technologies",
	[3] = "Buffalo (Formerly Melco)",
	[4] = "TriMedia Technologies",
	[5] = "Cyan Technologies",
	[6] = "Global Locate",
	[7] = "Optillion",
	[8] = "Terago Communications",
	[9] = "Ikanos Communications",
	[10] = "Princeton Technology",
	[11] = "Nanya Technology",
	[12] = "Elite Flash Storage",
	[13] = "Mysticom",
	[14] = "LightSand Communications",
	[15] = "ATI Technologies",
	[16] = "Agere Systems",
	[17] = "NeoMagic",
	[18] = "AuroraNetics",
	[19] = "Golden Empire",
	[20] = "Muskin",
	[21] = "Tioga Technologies",
	[22] = "Netlist",
	[23] = "TeraLogic",
	[24] = "Cicada Semiconductor",
	[25] = "Centon Electronics",
	[26] = "Tyco Electronics",
	[27] = "Magis Works",
	[28] = "Zettacom",
	[29] = "Cogency Semiconductor",
	[30] = "Chipcon AS",
	[31] = "Aspex Technology",
	[32] = "F5 Networks",
	[33] = "Programmable Silicon Solutions",
	[34] = "ChipWrights",
	[35] = "Acorn Networks",
	[36] = "Quicklogic",
	[37] = "Kingmax Semiconductor",
	[38] = "BOPS",
	[39] = "Flasys",
	[40] = "BitBlitz Communications",
	[41] = "eMemory Technology",
	[42] = "Procket Networks",
	[43] = "Purple Ray",
	[44] = "Trebia Networks",
	[45] = "Delta Electronics",
	[46] = "Onex Communications",
	[47] = "Ample Communications",
	[48] = "Memory Experts Intl",
	[49] = "Astute Networks",
	[50] = "Azanda Network Devices",
	[51] = "Dibcom",
	[52] = "Tekmos",
	[53] = "API NetWorks",
	[54] = "Bay Microsystems",
	[55] = "Firecron Ltd",
	[56] = "Resonext Communications",
	[57] = "Tachys Technologies",
	[58] = "Equator Technology",
	[59] = "Concept Computer",
	[60] = "SILCOM",
	[61] = "3Dlabs",
	[62] = "ct Magazine",
	[63] = "Sanera Systems",
	[64] = "Silicon Packets",
	[65] = "Viasystems Group",
	[66] = "Simtek",
	[67] = "Semicon Devices Singapore",
	[68] = "Satron Handelsges",
	[69] = "Improv Systems",
	[70] = "INDUSYS GmbH",
	[71] = "Corrent",
	[72] = "Infrant Technologies",
	[73] = "Ritek Corp",
	[74] = "empowerTel Networks",
	[75] = "Hypertec",
	[76] = "Cavium Networks",
	[77] = "PLX Technology",
	[78] = "Massana Design",
	[79] = "Intrinsity",
	[80] = "Valence Semiconductor",
	[81] = "Terawave Communications",
	[82] = "IceFyre Semiconductor",
	[83] = "Primarion",
	[84] = "Picochip Designs Ltd",
	[85] = "Silverback Systems",
	[86] = "Jade Star Technologies",
	[87] = "Pijnenburg Securealink",
	[88] = "TakeMS International AG",
	[89] = "Cambridge Silicon Radio",
	[90] = "Swissbit",
	[91] = "Nazomi Communications",
	[92] = "eWave System",
	[93] = "Rockwell Collins",
	[94] = "Picocel Co., Ltd.",
	[95] = "Alphamosaic Ltd",
	[96] = "Sandburst",
	[97] = "SiCon Video",
	[98] = "NanoAmp Solutions",
	[99] = "Ericsson Technology",
	[100] = "PrairieComm",
	[101] = "Mitac International",
	[102] = "Layer N Networks",
	[103] = "MtekVision (Atsana)",
	[104] = "Allegro Networks",
	[105] = "Marvell Semiconductors",
	[106] = "Netergy Microelectronic",
	[107] = "NVIDIA",
	[108] = "Internet Machines",
	[109] = "Peak Electronics",
	[111] = "Accton Technology",
	[112] = "Teradiant Networks",
	[113] = "Europe Technologies",
	[114] = "Cortina Systems",
	[115] = "RAM Components",
	[116] = "Raqia Networks",
	[117] = "ClearSpeed",
	[118] = "Matsushita Battery",
	[119] = "Xelerated",
	[120] = "SimpleTech",
	[121] = "Utron Technology",
	[122] = "Astec International",
	[123] = "AVM gmbH",
	[124] = "Redux Communications",
	[125] = "Dot Hill Systems",
	[126] = "TeraChip",
};

static const char *jedec_id5[] = {
	[1] = "T-RAM Incorporated",
	[2] = "Innovics Wireless",
	[3] = "Teknovus",
	[4] = "KeyEye Communications",
	[5] = "Runcom Technologies",
	[6] = "RedSwitch",
	[7] = "Dotcast",
	[8] = "Silicon Mountain Memory",
	[9] = "Signia Technologies",
	[10] = "Pixim",
	[11] = "Galazar Networks",
	[12] = "White Electronic Designs",
	[13] = "Patriot Scientific",
	[14] = "Neoaxiom Corporation",
	[15] = "3Y Power Technology",
	[16] = "Europe Technologies",
	[17] = "Potentia Power Systems",
	[18] = "C-guys Incorporated",
	[19] = "Digital Communications Technology Incorporated",
	[20] = "Silicon-Based Technology",
	[21] = "Fulcrum Microsystems",
	[22] = "Positivo Informatica Ltd",
	[23] = "XIOtech Corporation",
	[24] = "PortalPlayer",
	[25] = "Zhiying Software",
	[26] = "Direct2Data",
	[27] = "Phonex Broadband",
	[28] = "Skyworks Solutions",
	[29] = "Entropic Communications",
	[30] = "Pacific Force Technology",
	[31] = "Zensys A/S",
	[32] = "Legend Silicon Corp.",
	[33] = "sci-worx GmbH",
	[34] = "Oasis Silicon Systems",
	[35] = "Renesas Technology",
	[36] = "Raza Microelectronics",
	[37] = "Phyworks",
	[38] = "MediaTek",
	[39] = "Non-cents Productions",
	[40] = "US Modular",
	[41] = "Wintegra Ltd",
	[42] = "Mathstar",
	[43] = "StarCore",
	[44] = "Oplus Technologies",
	[45] = "Mindspeed",
	[46] = "Just Young Computer",
	[47] = "Radia Communications",
	[48] = "OCZ",
	[49] = "Emuzed",
	[50] = "LOGIC Devices",
	[51] = "Inphi Corporation",
	[52] = "Quake Technologies",
	[53] = "Vixel",
	[54] = "SolusTek",
	[55] = "Kongsberg Maritime",
	[56] = "Faraday Technology",
	[57] = "Altium Ltd.",
	[58] = "Insyte",
	[59] = "ARM Ltd.",
	[60] = "DigiVision",
	[61] = "Vativ Technologies",
	[62] = "Endicott Interconnect Technologies",
	[63] = "Pericom",
	[64] = "Bandspeed",
	[65] = "LeWiz Communications",
	[66] = "CPU Technology",
	[67] = "Ramaxel Technology",
	[68] = "DSP Group",
	[69] = "Axis Communications",
	[70] = "Legacy Electronics",
	[71] = "Chrontel",
	[72] = "Powerchip Semiconductor",
	[73] = "MobilEye Technologies",
	[74] = "Excel Semiconductor",
	[75] = "A-DATA Technology",
	[76] = "VirtualDigm",
	[77] = "G Skill Intl",
	[78] = "Quanta Computer",
	[79] = "Yield Microelectronics",
	[80] = "Afa Technologies",
	[81] = "KINGBOX Technology Co. Ltd.",
	[82] = "Ceva",
	[83] = "iStor Networks",
	[84] = "Advance Modules",
	[85] = "Microsoft",
	[86] = "Open-Silicon",
	[87] = "Goal Semiconductor",
	[104] = "ARC International",
	[89] = "Simmtec",
	[90] = "Metanoia",
	[91] = "Key Stream",
	[92] = "Lowrance Electronics",
	[93] = "Adimos",
	[94] = "SiGe Semiconductor",
	[95] = "Fodus Communications",
	[96] = "Credence Systems Corp.",
	[97] = "Genesis Microchip Inc.",
	[98] = "Vihana, Inc.",
	[99] = "WIS Technologies",
	[100] = "GateChange Technologies",
	[101] = "High Density Devices AS",
	[102] = "Synopsys",
	[103] = "Gigaram",
	[104] = "Enigma Semiconductor Inc.",
	[105] = "Centry Micro Inc.",
	[106] = "Icera Semiconductor",
	[107] = "Mediaworks Integrated Systems",
	[108] = "O'Neil Product Development",
	[109] = "Supreme Top Technology Ltd.",
	[110] = "MicroDisplay Corporation",
	[111] = "Team Group Inc.",
	[112] = "Sinett Corporation",
	[113] = "Toshiba Corporation",
	[114] = "Tensilica",
	[115] = "SiRF Technology",
	[116] = "Bacoc Inc.",
	[117] = "SMaL Camera Technologies",
	[118] = "Thompson SC",
	[119] = "Airgo Networks",
	[120] = "Wisair Ltd.",
	[121] = "SigmaTel",
	[122] = "Arkados",
	[123] = "Compete IT gmbH Co. KG",
	[124] = "Eudar Technology Inc.",
	[125] = "Focus Enhancements",
	[126] = "Xyratex",
};

static const char *jedec_id6[] = {
	[1] = "Specular Networks",
	[2] = "Patriot Memory",
	[3] = "U-Chip Technology Corp",
	[4] = "Silicon Optix",
	[5] = "Greenfield Networks",
	[6] = "CompuRAM GmbH",
	[7] = "Stargen, Inc.",
	[8] = "NetCell Corporation",
	[9] = "Excalibrus Technolgoies Ltd",
	[10] = "SCM Microsystems",
	[11] = "Xsigo Systems, Inc.",
	[12] = "CHIPS & Systems Inc",
	[13] = "Tier 1 Multichip Solutions",
	[14] = "CWRL Labs",
	[15] = "Teradici",
	[16] = "Gigaram, Inc.",
	[17] = "g2 Microsystems",
	[18] = "PowerFlash Semiconductor",
	[19] = "P.A. Semi, Inc.",
	[20] = "NovaTech Solutions, S.A.",
	[21] = "c2 Microsystems, Inc.",
	[22] = "Level5 Networks",
	[23] = "COS Memory AG",
	[24] = "Innovasic Semiconductor",
	[25] = "O2IC Co. Ltd",
	[26] = "Tabula, Inc.",
	[27] = "Crucial Technology",
	[28] = "Chelsio Communications",
	[29] = "Solarflare Communications",
	[30] = "Xambala Inc.",
	[31] = "EADS Astrium",
	[32] = "ATO Semicon Co. Ltd.",
	[33] = "Imaging Works, Inc.",
	[34] = "Astute Networks, Inc.",
	[35] = "Tzero",
	[36] = "Emulex",
	[37] = "Power-One",
	[38] = "Pulse~LINK Inc.",
	[39] = "Hon Hai Precision Industry",
	[40] = "White Rock Networks Inc.",
	[41] = "Telegent Systems USA, Inc.",
	[42] = "Atrua Technologies, Inc.",
	[43] = "Acbel Polytech Inc.",
	[44] = "eRide Inc.",
	[45] = "ULi Electronics Inc.",
	[46] = "Magnum Semiconductor Inc.",
	[47] = "neoOne Technology, Inc.",
	[48] = "Connex Technology, Inc.",
	[49] = "Stream Processors, Inc.",
	[50] = "Focus Enhancements",
	[51] = "Telecis Wireless, Inc.",
	[52] = "uNav Microelectronics",
	[53] = "Tarari, Inc.",
	[54] = "Ambric, Inc.",
	[55] = "Newport Media, Inc.",
	[56] = "VMTS",
	[57] = "Enuclia Semiconductor, Inc.",
	[58] = "Virtium Technology Inc.",
	[59] = "Solid State System Co., Ltd.",
	[60] = "Kian Tech LLC",
	[61] = "Artimi",
	[62] = "Power Quotient International",
	[63] = "Avago Technologies",
	[64] = "ADTechnology",
	[65] = "Sigma Designs",
	[66] = "SiCortex, Inc.",
	[67] = "Ventura Technology Group",
	[68] = "eASIC",
	[69] = "M.H.S SAS",
	[70] = "Micro Star International",
	[71] = "Rapport Inc.",
	[72] = "Makway International",
	[73] = "Broad Reach Engineering Co.",
	[74] = "Semiconductor Mfg Intl Corp",
	[75] = "SiConnect",
	[76] = "FCI USA Inc.",
	[77] = "Validity Sensors",
	[78] = "Coney Technology Co. Ltd.",
	[79] = "Spans Logic",
	[80] = "Neterion Inc.",
	[81] = "Qimonda",
	[82] = "New Japan Radio Co. Ltd.",
	[83] = "Velogix",
	[84] = "Montalvo Systems",
	[85] = "iVivity Inc.",
	[86] = "Walton Chaintech",
	[87] = "AENEON",
	[88] = "Lorom Industrial Co. Ltd.",
	[89] = "Radiospire Networks",
	[90] = "Sensio Technologies, Inc.",
	[91] = "Nethra Imaging",
	[92] = "Hexon Technology Pte Ltd",
	[93] = "CompuStocx (CSX)",
	[94] = "Methode Electronics, Inc.",
	[95] = "Connect One Ltd.",
	[96] = "Opulan Technologies",
	[97] = "Septentrio NV",
	[98] = "Goldenmars Technology Inc.",
	[99] = "Kreton Corporation",
	[100] = "Cochlear Ltd.",
	[101] = "Altair Semiconductor",
	[102] = "NetEffect, Inc.",
	[103] = "Spansion, Inc.",
	[104] = "Taiwan Semiconductor Mfg",
	[105] = "Emphany Systems Inc.",
	[106] = "ApaceWave Technologies",
	[107] = "Mobilygen Corporation",
	[108] = "Tego",
	[109] = "Cswitch Corporation",
	[110] = "Haier (Beijing) IC Design Co.",
	[111] = "MetaRAM",
	[112] = "Axel Electronics Co. Ltd.",
};

static const char **jedec_ids[] = {
	jedec_id1,
	jedec_id2,
	jedec_id3,
	jedec_id4,
	jedec_id5,
	jedec_id6,
};

/*
 * jedec_manufacturer  -  find manufacturer based on id and continuation bytes
 *
 * @idx:        continuation byte index
 * @id:         identifier
 *
 * returns pointer to manufacturer id
 */
const char *jedec_manufacturer(int idx, uint8_t id)
{
	if (0 <= idx &&
            idx < sizeof(jedec_ids) / sizeof(jedec_ids[0]))
		return jedec_ids[idx][id];
	return NULL;
}
